/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "main.h"
#if (GTK_MINOR_VERSION >= 10)

#include "gui_status_icon.h"

static gboolean
status_icon_activate (GtkStatusIcon *status_icon, MainPrivate* priv)
{
	if(GTK_WIDGET_VISIBLE(priv->window)){
		gtk_window_get_position(GTK_WINDOW (priv->window), &priv->window_x, &priv->window_y);
		gtk_widget_hide_all (priv->window);
	}else{		
		gtk_widget_show_all (priv->window);
		gtk_window_move(GTK_WINDOW (priv->window), priv->window_x, priv->window_y);
	}

	return FALSE;
}

static gboolean
status_icon_popup_menu (GtkStatusIcon *icon, guint button, guint time, MainPrivate* priv)
{
	/* TODO Add menu */
	return TRUE;
}

GtkStatusIcon*
gui_status_icon_create(MainPrivate* priv, gboolean init_status)
{
	GtkStatusIcon* status;
	gchar* filename;

	if(init_status)
		filename = g_strconcat(PACKAGE_DATA_DIR, STATUS_ICON_ONLINE, NULL);
	else
		filename = g_strconcat(PACKAGE_DATA_DIR, STATUS_ICON_OFFLINE, NULL);

	status = gtk_status_icon_new_from_file(filename);

	g_signal_connect (status, "popup-menu",
					G_CALLBACK (status_icon_popup_menu), priv);

	g_signal_connect (status, "activate",
					G_CALLBACK (status_icon_activate), priv);

	gtk_status_icon_set_visible(status, TRUE);

	g_free(filename);

	return status;
}

void
gui_status_icon_set_status(GtkStatusIcon* status_icon, gboolean status)
{
	gchar* filename;

	if(status)
		filename = g_strconcat(PACKAGE_DATA_DIR, STATUS_ICON_ONLINE, NULL);
	else
		filename = g_strconcat(PACKAGE_DATA_DIR, STATUS_ICON_OFFLINE, NULL);

	gtk_status_icon_set_from_file (status_icon, filename);

	g_free(filename);
}

#endif
