/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /** fork() */
#include "io_misc.h"

#define MAX_LINE_SIZE 128

gchar* 
io_misc_get_addr()
{
	FILE* pfile;
	gchar* temp;
	gchar* path;
	gchar* addr = NULL;

	path = g_strconcat(getenv("HOME"), HAMACHI_STATE_FILE, NULL);

	pfile = fopen(path, "r");

	if(pfile == NULL){
		perror("fopen()");
		g_free(path);
		return NULL;
	}

	temp = g_malloc0(MAX_LINE_SIZE);

	while(fgets(temp, MAX_LINE_SIZE, pfile) != NULL){
		if(g_strrstr(temp, "Identity") != NULL){			
			if(sscanf(temp, "Identity\t%as", &addr) != 1){
				g_printerr("Error in scanf\n");
			}
		}
	}

	g_free(temp);
	g_free(path);
	fclose(pfile);
	
	return addr;
}

void
io_misc_run_command(gchar* cmd)  
{	
	gint err;
	pid_t fork_res;

	fork_res = fork();

	if(fork_res == -1){
			perror("io_misc_run_command - fork()");
			return;
	}

	if(fork_res == 0){ /** This is a child */
			err = execlp("/bin/sh", "/bin/sh", "-c", cmd, NULL);
			if(err == -1){
					perror("io_misc_run_command - Error in execlp");
			}
	}else{ /** This is the parent */
			/** We could wait for the child here, but we want it do be non-blocking */
			/** We will wait for the pid in the io_update_loop... */
	}
}

