/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h> 	/** waitpid()*/
#include <sys/wait.h> 	/** waitpid()*/

#include "gui_top_area.h"
#include "gui_bottom_area.h"
#include "gui_middle_area.h"
#include "io_misc.h"
#include "io_update.h"
#include "io_ipc.h"
#include "main.h"

enum errors{
  	HAMACHI_NOT_STARTED,
  	TUNCFG_NOT_STARTED,
  	NO_ERROR
};

static gint
fix_errors(gint err)
{
	FILE* out;
	gchar msg[512];
	gint count;

	switch(err){
	case HAMACHI_NOT_STARTED:
		out = popen("hamachi start", "r");
		count = fread(&msg, 1, 512, out);
		pclose(out);

		if(count > 0){
			if(g_strrstr((gchar*)&msg, "tap: connect()") != NULL)
				return TUNCFG_NOT_STARTED;
		}else{
			return NO_ERROR;
		}
	case TUNCFG_NOT_STARTED:
		out = popen("gksudo tuncfg", "r");
		pclose(out);
		break;

	default:
		break;
	}

	return NO_ERROR;
}

void
io_update_loop (MainPrivate* priv)
{
	gint err = HAMACHI_NOT_STARTED;
	gchar* path = NULL;	
	gchar* addr;
	IPCStatus* stat = NULL;
	IPCNetwork *old_net = NULL;	
	IPCNetwork *new_net = NULL;
	
	if(priv->connection == NULL){
		while(priv->hamchi_user_stop)
			g_usleep(G_USEC_PER_SEC * 1);
		
		path = ipc_get_sock_path();	
		addr = io_misc_get_addr();	
		priv->connection = ipc_connect(path);
		
		if(priv->connection == NULL){
			while(err != NO_ERROR){
				if(err == HAMACHI_NOT_STARTED){
					gdk_threads_enter();
					gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), "Starting Hamachi...");
					gdk_threads_leave();
				}else if(err == TUNCFG_NOT_STARTED){
					gdk_threads_enter();
					gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), "Starting Tuncfg...");
					gdk_threads_leave();					
				}
				
				err = fix_errors(err);
				g_usleep(G_USEC_PER_SEC * 1);
			}
			io_update_loop(priv);
		}
		
		stat = ipc_status_get(priv->connection);
		gdk_threads_enter();
		gui_top_area_set_addr(GUI_TOP_AREA(priv->top_area), addr);
		gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), stat->nick);
		gdk_threads_leave();
		g_free(path);
	}
	
	for(;;){
		if(priv->connection == NULL){
			io_update_loop(priv);
			return;
		}		
		
		new_net = ipc_list_get(priv->connection);
		
		if(priv->connection->error != IPC_ERROR_NO_ERROR){
			g_printerr("IPC error: %s\n", ipc_error_string(priv->connection));
			ipc_disconnect(priv->connection);
			priv->connection = NULL;
			io_update_loop(priv);
			return;
		}
		
		if(!ipc_list_equal(new_net, old_net)){		
			if(!ipc_list_nicks_check(new_net)){
				gdk_threads_enter();	
				gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), "Retrieving nicks...");
				gdk_threads_leave();
				ipc_list_nicks_get(priv->connection);
			}else{			
				gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), stat->nick);	
				gdk_threads_enter();
				gui_middle_area_set_network(GUI_MIDDLE_AREA(priv->middle_area), new_net);		
				gdk_threads_leave();
				ipc_list_free(old_net);
				old_net = new_net;
			}
		}
		err = waitpid(-1, NULL, WNOHANG);
		if(err > 0)
			g_printerr("Child with pid %d, is now dead\n", err);
		g_usleep(G_USEC_PER_SEC * 2);	
	}
	/**
	gint err;
	gint status;
	gint timeout = IO_STARTUP_DELAY;

	gchar* nick = NULL;
	gchar* addr = NULL;

	node* nod = NULL;
	queue_cont* cont;

	while(queue == NULL){
		sleep(1);
	}

	g_async_queue_ref(queue);

	for(;;){
		sleep(timeout);

		nod = ipc_get_list(&err);

		if(err != IPC_ERR_NO){
			switch(err){
			case IPC_ERR_NICKS:
				io_update_set_error_nick(NICKS_NOT_RETRIEVED);
				ipc_get_nicks();
			break;
			case IPC_ERR_NO_UPDATE:
				break;
			case IPC_ERR_SOCKET:
				g_printerr("Error in ipc sock...\n");
				break;
			case IPC_ERR_TRANS:
				g_printerr("Error in ipc sock transmission...\n");
				break;
			case IPC_ERR_CONNECT:
				if(!priv->hamchi_user_stop){
					if(fix_errors(HAMACHI_NOT_STARTED) == TUNCFG_NOT_STARTED){
						io_update_set_error_nick(TUNCFG_NOT_STARTED);
						fix_errors(TUNCFG_NOT_STARTED);
					}else{
						io_update_set_error_nick(HAMACHI_NOT_STARTED);
					}
				}
				break;
			default:
				break;
			}
		}else{

			nick = g_malloc0(50);
			addr = g_malloc0(50);

			if(io_identity_get(nick, addr) == -1){
				g_free(nick);
				g_free(addr);

				cont = g_malloc0(sizeof(queue_cont));
				cont->data = nod;
				cont->type = NODE;

				g_async_queue_push (queue, cont);
			}else{
		               	cont = g_malloc0(sizeof(queue_cont));
				cont->data = nod;
				cont->type = NODE;
				g_async_queue_push (queue, cont);

				cont = g_malloc0(sizeof(queue_cont));
				cont->data = nick;
				cont->type = NICK;
				g_async_queue_push (queue, cont);

				cont = g_malloc0(sizeof(queue_cont));
				cont->data = addr;
				cont->type = ADDR;
				g_async_queue_push (queue, cont);
			}
		}
	*/
	/** Check if any of our childern has died... */
	/**
		err = waitpid(-1, &status, WNOHANG);
#ifdef DEBUG
		if(err > 0)
			g_printerr("Child with pid %d, is now dead\n", err);
#endif
	}
	*/
}
